# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING

from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import WorkloadsMgmtClientConfiguration
from .operations import (
    MonitorsOperations,
    Operations,
    ProviderInstancesOperations,
    SAPApplicationServerInstancesOperations,
    SAPCentralInstancesOperations,
    SAPDatabaseInstancesOperations,
    SAPVirtualInstancesOperations,
    SapLandscapeMonitorOperations,
    WorkloadsMgmtClientOperationsMixin,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class WorkloadsMgmtClient(
    WorkloadsMgmtClientOperationsMixin
):  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Workloads client provides access to various workload operations.:code:`<br>`Azure Center for
    SAP solutions is currently in PREVIEW. See the `Azure Center for SAP solutions - Legal Terms
    <https://learn.microsoft.com/en-us/legal/azure-center-for-sap-solutions/azure-center-for-sap-solutions-legal-terms>`_
    for legal notices applicable to Azure Center for SAP solutions.

    :ivar sap_virtual_instances: SAPVirtualInstancesOperations operations
    :vartype sap_virtual_instances:
     azure.mgmt.workloads.aio.operations.SAPVirtualInstancesOperations
    :ivar sap_central_instances: SAPCentralInstancesOperations operations
    :vartype sap_central_instances:
     azure.mgmt.workloads.aio.operations.SAPCentralInstancesOperations
    :ivar sap_database_instances: SAPDatabaseInstancesOperations operations
    :vartype sap_database_instances:
     azure.mgmt.workloads.aio.operations.SAPDatabaseInstancesOperations
    :ivar sap_application_server_instances: SAPApplicationServerInstancesOperations operations
    :vartype sap_application_server_instances:
     azure.mgmt.workloads.aio.operations.SAPApplicationServerInstancesOperations
    :ivar monitors: MonitorsOperations operations
    :vartype monitors: azure.mgmt.workloads.aio.operations.MonitorsOperations
    :ivar provider_instances: ProviderInstancesOperations operations
    :vartype provider_instances: azure.mgmt.workloads.aio.operations.ProviderInstancesOperations
    :ivar sap_landscape_monitor: SapLandscapeMonitorOperations operations
    :vartype sap_landscape_monitor:
     azure.mgmt.workloads.aio.operations.SapLandscapeMonitorOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.workloads.aio.operations.Operations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2023-04-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = WorkloadsMgmtClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        self._client = AsyncARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.sap_virtual_instances = SAPVirtualInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.sap_central_instances = SAPCentralInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.sap_database_instances = SAPDatabaseInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.sap_application_server_instances = SAPApplicationServerInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.monitors = MonitorsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.provider_instances = ProviderInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.sap_landscape_monitor = SapLandscapeMonitorOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)

    def _send_request(self, request: HttpRequest, **kwargs: Any) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "WorkloadsMgmtClient":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
